@@ -146,9 +146,9 @@ def harmonize_np(ws_files, reference, input_config_path, base_path, outfile):
     """
     from quickstats.components import NuisanceParameterHarmonizer
     harmonizer = NuisanceParameterHarmonizer(reference)
-    if (len(ws_files) > 0) and input_list is not None:
+    if (len(ws_files) > 0) and input_config_path is not None:
         raise RuntimeError('either workspace paths or json file containing workspace paths should be given')
     if len(ws_files) > 0:
         harmonizer.harmonize(ws_files, outfile=outfile)
-    elif (input_list is not None):
+    elif (input_config_path is not None):
         harmonizer.harmonize_multi_input(input_config_path, base_path, outfile=outfile) 
\ No newline at end of file
