@@ -43,7 +43,7 @@ class NuisanceParameterHarmonizer(object):
             regex = re.compile(pattern)
             matches += [s for s in source if regex.search(s)]
         if excludes is not None:
-            anti_matches = WSNuisParamHarmonizer.filter_nuis(source, excludes)
+            anti_matches = NuisanceParameterHarmonizer.filter_nuis(source, excludes)
             matches = list(set(matches) - set(anti_matches))
         return matches
     
@@ -77,10 +77,10 @@ class NuisanceParameterHarmonizer(object):
                 continue
             print('{indent}{description}:'.format(indent='\t'*level, description=description))
             if 'source' in reference[k]:
-                WSNuisParamHarmonizer.display_mappings(v, reference[k]['source'], level=1)
+                NuisanceParameterHarmonizer.display_mappings(v, reference[k]['source'], level=1)
             else:
                 for old_name, new_name in v.items():
-                    WSNuisParamHarmonizer.display_difference(old_name, new_name)
+                    NuisanceParameterHarmonizer.display_difference(old_name, new_name)
                 
     @staticmethod
     def reshape_mappings(mappings, roots=[], result=None):
@@ -88,7 +88,7 @@ class NuisanceParameterHarmonizer(object):
             result = {}
         for key, value in mappings.items():
             if isinstance(value, dict):
-                WSNuisParamHarmonizer.reshape_mappings(value, roots + [key], result)
+                NuisanceParameterHarmonizer.reshape_mappings(value, roots + [key], result)
             elif key not in result:
                 result[key] = [(roots, value)]
             else:
@@ -99,7 +99,7 @@ class NuisanceParameterHarmonizer(object):
     def flatten_mappings(mappings):
         flattened_mappings = {}
         # check duplicates
-        reshaped_mappings = WSNuisParamHarmonizer.reshape_mappings(mappings)
+        reshaped_mappings = NuisanceParameterHarmonizer.reshape_mappings(mappings)
         for old_name in reshaped_mappings:
             dest = reshaped_mappings[old_name]
             if len(reshaped_mappings[old_name]) > 1:
@@ -130,10 +130,10 @@ class NuisanceParameterHarmonizer(object):
         for group in reference:
             selected = []
             # NPs that match a pattern with exclusion
-            selected += WSNuisParamHarmonizer.filter_nuis(nuis_names, reference[group].get('pattern', None), 
+            selected += NuisanceParameterHarmonizer.filter_nuis(nuis_names, reference[group].get('pattern', None), 
                                                           reference[group].get('exclude', None))
             # NPs that match exactly
-            selected += WSNuisParamHarmonizer.filter_nuis(nuis_names, reference[group].get('match', None))
+            selected += NuisanceParameterHarmonizer.filter_nuis(nuis_names, reference[group].get('match', None))
             if not selected:
                 continue
             if ('source' not in reference[group]) and ('names' not in reference[group]):
@@ -155,13 +155,13 @@ class NuisanceParameterHarmonizer(object):
                     names = []
                     for key, value in reference[group]['source'].items():
                         names += value['names']
-                suggested_names = WSNuisParamHarmonizer.get_suggested_names(selected, names, cutoff)
+                suggested_names = NuisanceParameterHarmonizer.get_suggested_names(selected, names, cutoff)
                 if suggested_names:
                     if 'names' in reference[group]:
                         mappings[group] = suggested_names
                     elif 'source' in reference[group]:
                         source = reference[group]['source']
-                        mappings[group] = WSNuisParamHarmonizer.match_source(suggested_names, source)
+                        mappings[group] = NuisanceParameterHarmonizer.match_source(suggested_names, source)
         return mappings
 
 
@@ -173,14 +173,14 @@ class NuisanceParameterHarmonizer(object):
         if len(unknown_nuis) > 0:
             print('Remained NPs:')
             for unknown in unknown_nuis:
-                WSNuisParamHarmonizer.display_difference(unknown, '?')
+                NuisanceParameterHarmonizer.display_difference(unknown, '?')
         flattened_mappings.update({np:None for np in unknown_nuis})
         return flattened_mappings
     
     def harmonize(self, ws_paths, outfile=None):
         if isinstance(ws_paths, str):
             ws_paths = [ws_paths]
-        nuis_names = WSNuisParamHarmonizer.get_nuiance_parameter_names_collection(ws_paths)
+        nuis_names = NuisanceParameterHarmonizer.get_nuiance_parameter_names_collection(ws_paths)
         harmonize_map = self.get_harmonize_map(nuis_names)
         if outfile is not None:
             json.dump(harmonize_map, open(outfile, 'w'), indent=2)
@@ -213,8 +213,8 @@ class NuisanceParameterHarmonizer(object):
             raise ValueError('invalid input expr')
         
     def harmonize_multi_input(self, input_config_path, base_path='./', outfile=None):
-        config = json.load(open(input_path))
-        input_paths = WSNuisParamHarmonizer.get_input_paths(config, base_path)
+        config = json.load(open(input_config_path))
+        input_paths = NuisanceParameterHarmonizer.get_input_paths(config, base_path)
         harmonize_map = self._harmonize_multi_input(input_paths)
         if outfile is not None:
             json.dump(harmonize_map, open(outfile, 'w'), indent=2)
