@@ -1,221 +1,24 @@
+##################################################################################################
+# Based on https://gitlab.cern.ch/atlas-physics/stat/tools/StatisticsTools by Stefan Gadatsch
+# Author: Alkaid Cheng
+# Email: chi.lung.cheng@cern.ch
+##################################################################################################
 import os
+import sys
 import time
 import json
 import fnmatch
+from typing import List, Optional
 
 import uproot
 import numpy as np
 
-from quickstats.components import CLIParser, ExtendedModel, ExtendedMinimizer
+from quickstats.components import ExtendedModel, ExtendedMinimizer
 from quickstats.utils import root_utils, common_utils
 
 import ROOT
 
 class NuisanceParameterPull(object):
-    _CLI_USAGE_          = 'quickstats run_pulls [-h|--help] [<args>]'
-    _CLI_DESCRIPTION_    = 'Tool for computing the impact of a given NP to a set of POIs'    
-    _CLI_ARG_OPTIONS_    = \
-        {
-            'input_file': {
-                'abbr'        : 'i',
-                'description' : 'Path to the input workspace file',
-                'required'    : True,
-                'type'        : str
-            },
-            'workspace':{
-                'abbr'        : 'w',
-                'description' : 'Name of workspace',
-                'required'    : False,
-                'type'        : str,
-                'default'     : 'combWS'
-            },
-            'model_config':{
-                'abbr'        : 'm',
-                'description' : 'Name of model config',
-                'required'    : False,
-                'type'        : str,
-                'default'     : 'ModelConfig'
-            },  
-            'data':{
-                'abbr'        : 'd',
-                'description' : 'Name of dataset',
-                'required'    : False,
-                'type'        : str,
-                'default'     : 'combData'
-            },
-            'parameter'  : {
-                'abbr'        : 'p',
-                'description' : 'Nuisance parameter(s) to run pulls on.' +\
-                                'Multiple parameters are separated by commas.' +\
-                                'Wildcards are accepted', 
-                'required'    : False,
-                'type'        : str,
-                'default'     : ''
-            },   
-            'poi': {
-                'abbr'        : 'x',
-                'description' : 'POIs to measure',
-                'required'    : False,
-                'type'        : str,
-                'default'     : ""
-            },
-            'profile': {
-                'abbr'        : 'r',
-                'description' : 'Parameters to profile', 
-                'required'    : False,
-                'type'        : str,
-                'default'     : ""
-            },
-            'fix': {
-                'abbr'        : 'f',
-                'description' : 'Parameters to fix', 
-                'required'    : False,
-                'type'        : str,
-                'default'     : ""
-            },
-            'snapshot': {
-                'abbr'        : 's',
-                'description' : 'Name of initial snapshot', 
-                'required'    : False,
-                'type'        : str,
-                'default'     : "nominalNuis"
-            },
-            'outdir': {
-                'abbr'        : 'o',
-                'description' : 'Output directory', 
-                'required'    : False,
-                'type'        : str,
-                'default'     : "output"
-            },
-            'minimizer_type'  : {
-                'abbr'        : 't',
-                'description' : 'Minimizer type', 
-                'required'    : False,
-                'type'        : str,
-                'default'     : "Minuit2"
-            },        
-            'minimizer_algo'  : {
-                'abbr'        : 'a',
-                'description' : 'Minimizer algorithm', 
-                'required'    : False,
-                'type'        : str,
-                'default'     : "Migrad"
-            },          
-            'num_cpu'  : {
-                'abbr'        : 'c',
-                'description' : 'Number of CPUs to use', 
-                'required'    : False,
-                'type'        : int,
-                'default'     : 1
-            },   
-            'binned'  : {
-                'abbr'        : 'b',
-                'description' : 'Binned likelihood', 
-                'required'    : False,
-                'type'        : int,
-                'default'     : 1
-            },    
-            'precision'  : {
-                'abbr'        : 'q',
-                'description' : 'Precision for scan', 
-                'required'    : False,
-                'type'        : float,
-                'default'     : 0.001
-            },          
-            'eps'  : {
-                'abbr'        : 'e',
-                'description' : 'Convergence criterium', 
-                'required'    : False,
-                'type'        : float,
-                'default'     : 1.0
-            },              
-            'log_level'  : {
-                'abbr'        : 'l',
-                'description' : 'Log level', 
-                'required'    : False,
-                'type'        : str,
-                'default'     : "INFO"
-            },      
-            'eigen'  : {
-                'abbr'        : None,
-                'description' : 'Compute eigenvalues and vectors', 
-                'required'    : False,
-                'type'        : int,
-                'default'     : 0
-            },   
-            'strategy'  : {
-                'abbr'        : None,
-                'description' : 'default strategy', 
-                'required'    : False,
-                'type'        : int,
-                'default'     : 0
-            },
-            'fix_cache'  : {
-                'abbr'        : None,
-                'description' : 'Fix StarMomentMorph cache', 
-                'required'    : False,
-                'type'        : int,
-                'default'     : 1
-            },    
-            'fix_multi'  : {
-                'abbr'        : None,
-                'description' : 'Fix MultiPdf level 2', 
-                'required'    : False,
-                'type'        : int,
-                'default'     : 1
-            },            
-            'offset'  : {
-                'abbr'        : None,
-                'description' : 'Offset likelihood', 
-                'required'    : False,
-                'type'        : int,
-                'default'     : 1
-            },  
-            'optimize'  : {
-                'abbr'        : None,
-                'description' : 'Optimize constant terms', 
-                'required'    : False,
-                'type'        : int,
-                'default'     : 2
-            },       
-            'max_calls'  : {
-                'abbr'        : None,
-                'description' : 'Maximum number of function calls', 
-                'required'    : False,
-                'type'        : int,
-                'default'     : -1
-            }, 
-            'max_iters'  : {
-                'abbr'        : None,
-                'description' : 'Maximum number of Minuit iterations', 
-                'required'    : False,
-                'type'        : int,
-                'default'     : -1
-            },       
-            'parallel'  : {
-                'abbr'        : None,
-                'description' : 'Parallelize job across different nuisance parameters using N workers.'+\
-                                'Use -1 for N_CPU workers.', 
-                'required'    : False,
-                'type'        : int,
-                'default'     : 0
-            },
-            'cache'  : {
-                'abbr'        : None,
-                'description' : 'Cache existing result', 
-                'required'    : False,
-                'type'        : bool,
-                'default'     : True
-            },
-            'exclude'  : {
-                'abbr'        : None,
-                'description' : 'Exclude NPs (wildcard is accepted)', 
-                'required'    : False,
-                'type'        : str,
-                'default'     : ''
-            }        
-        } 
-    
     @property
     def model(self):
         return self._model
@@ -256,17 +59,6 @@ class NuisanceParameterPull(object):
         self._pois                = None
         self._observables         = None
     
-    def get_parser(self, **kwargs):
-        parser = CLIParser(description=self._CLI_DESCRIPTION_,
-                           usage=self._CLI_USAGE_)
-        parser.load_argument_options(self._CLI_ARG_OPTIONS_)
-        return parser
-    
-    def run_parser(self, args=None):
-        parser = self.get_parser()
-        kwargs = vars(parser.parse_args(args))
-        self.run_pulls(**kwargs)
-    
     @staticmethod
     def evaluate_impact(model:ExtendedModel, minimizer:ExtendedMinimizer,
                         nuis, nominal_value, pois, minimizer_options, snapshot=None):
@@ -281,22 +73,29 @@ class NuisanceParameterPull(object):
             poi_values.append(poi.getVal())
         elapsed_time = time.time() - start_time
         return poi_values, elapsed_time
-    
+
     @staticmethod
-    def _run_pulls(input_file:str, workspace:str, model_config:str, data:str,
-                   snapshot:str, nuis_name:str, poi_name:str, 
+    def _run_pulls(input_file:str, data:str, snapshot:str, nuis_name:str, poi_name:str, 
                    fix_param:str='', profile_param:str='', binned:bool=True,
+                   workspace:Optional[str]=None, model_config:Optional[str]=None,
                    minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', 
                    precision:float=0.001, eps:float=1.0, strategy:int=0, 
                    num_cpu:int=1, offset:bool=True, optimize:bool=True, eigen:bool=False,
                    fix_cache:bool=True, fix_multi:bool=True, max_calls:int=-1, 
-                   max_iters:int=-1, outdir:str='output', cache=True, **kwargs):
+                   max_iters:int=-1, outdir:str='output', cache=True, logfile_path=None, **kwargs):
+        # create output directory if not exists
+        if not os.path.exists(outdir):
+            os.makedirs(outdir)        
         outname_root = os.path.join(outdir, nuis_name + '.root')
         outname_json = os.path.join(outdir, nuis_name + '.json')
         #checkpointing:
         if (os.path.exists(outname_root) and os.path.exists(outname_json)) and cache:
             print('INFO: Jobs already finished for the NP: {}. Skipping.'.format(nuis_name))
             return None
+        else:
+            print('INFO: Evaluating pulls for the NP: {}'.format(nuis_name))
+        if logfile_path is not None:
+            common_utils.redirect_stdout(logfile_path)
         start_time = time.time()
         # load model
         model = ExtendedModel(fname=input_file, ws_name=workspace,
@@ -427,11 +226,6 @@ class NuisanceParameterPull(object):
             print('INFO: Unconditional minimum of POI {}: {}'.format(rank_poi.GetName(), pois_hat[i]))
             print('INFO: POI when varying NP up by 1 sigma postfit (prefit): {} ({})'.format(pois_up[i], pois_nom_up[i]))
             print('INFO: POI when varying NP down by 1 sigma postfit (prefit): {} ({})'.format(pois_down[i], pois_nom_down[i]))
-
-        # create output directory if not exists
-        pulls_out_dir = os.path.join(outdir, 'pulls')
-        if not os.path.exists(outdir):
-            os.makedirs(outdir)
             
         # store result in root file
         outname_root = os.path.join(outdir, nuis_name + '.root')
@@ -466,23 +260,23 @@ class NuisanceParameterPull(object):
         outname_json = os.path.join(outdir, nuis_name + '.json')
         json.dump(result, open(outname_json, 'w'), indent=2)
         
+        if logfile_path is not None:
+            common_utils.restore_stdout()
 
-        
-    def run_pulls(self, input_file:str='workspace.root', workspace:str='combWS', model_config:str='ModelConfig',
-                  data:str='combData', poi:str='', snapshot:str='nominalNuis', outdir:str='output', profile:str="",
-                  fix:str='', minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', num_cpu:int=1, 
+    @common_utils.timer
+    def run_pulls(self, input_file:str='workspace.root', data:str='combData', poi:str='', 
+                  snapshot:str='nominalNuis', outdir:str='output', profile:str='',
+                  fix:str='', workspace:Optional[str]=None, model_config:Optional[str]=None,
+                  minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', num_cpu:int=1, 
                   binned:bool=1, precision:float=0.001, eps:float=1.0, log_level:str='INFO', eigen:bool=False, 
                   strategy:int=0, fix_cache:bool=True, fix_multi:bool=True, offset:bool=True, optimize:bool=True,
                   parameter:str="", max_calls:int=-1, max_iters:int=-1, parallel:int=0, cache:bool=True, 
                   exclude:str='', **kwargs):
         
-        start_time = time.time()
-        
         # configure default minimizer options
         ExtendedMinimizer._configure_default_minimizer_options(minimizer_type, minimizer_algo,
             strategy, debug_mode=(log_level=="DEBUG"))
         
-        
         nuis_list = ExtendedModel.get_nuisance_parameter_names(input_file, workspace, model_config)
         include_patterns = parameter.split(',')
         if parameter:
@@ -500,27 +294,30 @@ class NuisanceParameterPull(object):
                 if any(fnmatch.fnmatch(nuis_name, exclude_pattern) for exclude_pattern in exclude_patterns):
                     nuis_to_exclude.append(nuis_name)
         nuis_names = list(set(nuis_to_include) - set(nuis_to_exclude))
-                
+        
+        if log_level == 'INFO':
+            logfile_paths = [os.path.join(outdir, '{}.log'.format(nuis_name)) for nuis_name in nuis_names]
+        else:
+            logfile_paths = [None]*len(nuis_names)
+            
         if parallel:
             from itertools import repeat
-            arguments = (repeat(input_file), repeat(workspace), repeat(model_config),
-                         repeat(data), repeat(snapshot), nuis_names, repeat(poi),
-                         repeat(fix), repeat(profile), repeat(binned),
-                         repeat(minimizer_type), repeat(minimizer_algo), repeat(precision),
-                         repeat(eps), repeat(strategy), repeat(num_cpu), repeat(offset),
-                         repeat(optimize), repeat(eigen), repeat(fix_cache), repeat(fix_multi),
-                         repeat(max_calls), repeat(max_iters), repeat(outdir))
+            arguments = (repeat(input_file), repeat(data), repeat(snapshot), nuis_names, 
+                         repeat(poi), repeat(fix), repeat(profile), repeat(binned),
+                         repeat(workspace), repeat(model_config), repeat(minimizer_type), 
+                         repeat(minimizer_algo), repeat(precision), repeat(eps), repeat(strategy), 
+                         repeat(num_cpu), repeat(offset), repeat(optimize), repeat(eigen), 
+                         repeat(fix_cache), repeat(fix_multi), repeat(max_calls), repeat(max_iters), 
+                         repeat(outdir), repeat(cache), logfile_paths)
             max_workers = parallel if parallel != -1 else common_utils.get_cpu_count()
             common_utils.parallel_run(self._run_pulls, *arguments, max_workers=max_workers)
         else:
-            for nuis_name in nuis_names:
-                self._run_pulls(input_file, workspace, model_config, data, snapshot,
-                                nuis_name, poi, fix, profile, binned, minimizer_type, 
+            for i, nuis_name in enumerate(nuis_names):
+                self._run_pulls(input_file, data, snapshot, nuis_name, poi, fix, profile, 
+                                binned, workspace, model_config, minimizer_type, 
                                 minimizer_algo, precision, eps, strategy, num_cpu, offset, 
                                 optimize, eigen, fix_cache, fix_multi, max_calls, max_iters,
-                                outdir, cache)
-        end_time = time.time()
-        print('INFO: All jobs have finished. Total Time taken: {:.3f} s'.format(end_time-start_time))
+                                outdir, cache, logfile_paths[i])
         
     @staticmethod
     def parse_root_result(fname):
