@@ -1,9 +1,32 @@
+import os
 import time
 from six import string_types
 
 import numpy as np
 
 import ROOT
+import quickstats
+
+def compile_macro(name='RooTwoSidedCBShape'):
+    macros_dir = os.path.join(quickstats.macro_path, 'macros')
+    macros_path = os.path.join(macros_dir, name + '.cxx')
+    print('INFO: Compiling macro "{}"...'.format(name))
+    if not os.path.exists(macros_path):
+        print('ERROR: Cannot locate macro files from {}. ROOT macros will not be compiled.'.format(macros_dir))
+        return -1
+    return ROOT.gROOT.LoadMacro('{}++'.format(macros_path))
+
+def load_macro(name='RooTwoSidedCBShape'):  
+    macros_dir = os.path.join(quickstats.macro_path, 'macros')
+    macros_path = os.path.join(macros_dir, name + '_cxx')
+    if not os.path.exists(macros_path + '.so'):
+        return compile_macro(name)
+    result = ROOT.gSystem.Load(macros_path)
+    if result != 0:
+        raise RuntimeError("ERROR: Shared library for the macro {} is incompatible with the current pyROOT version. "
+                           "Please recompile by typing \"quickstats compile\".".format(name))
+    return result
+
 
 def print_resource_used(start):
     proc_info = ROOT.ProcInfo_t()
